//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <cassert>
#include <cstdlib>
#include <iostream>

template<typename T>
struct BasicAllocator
{
	static T* Allocate(unsigned int _n) { return static_cast<T*>(malloc(_n*sizeof(T))); }
	static void Desallocate(T* _ptr) { free(reinterpret_cast<void*>(_ptr)); }
};
//-----------------------------------------------------------------------------
template<typename T, template<typename> class A >
class Array
{
	public:
		Array(unsigned int _size=1);
		~Array();
		T& operator[](unsigned int _i);
		void push_back(const T& _val);
		unsigned int size() const;
		unsigned int capacity() const;

	private:
		typedef A<T> Allocator;
		T* values;
		unsigned int storageCapacity;
		unsigned int last;
};


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
template<typename T, template<typename> class A>
Array<T,A>::~Array()
{
	Allocator::Desallocate(values);
}
//-----------------------------------------------------------------------------
template<typename T, template<typename> class A >
Array<T,A>::Array(unsigned int _size):storageCapacity(_size),last(-1)
{
	storageCapacity = storageCapacity==0?1:storageCapacity;
	values 	= Allocator::Allocate(storageCapacity);
}
//-----------------------------------------------------------------------------
template<typename T, template<typename> class A >
T& Array<T,A>::operator[](unsigned int _i)
{
	assert(_i>=0);
	assert(_i<=last);
	return values[_i];
}
//-----------------------------------------------------------------------------
template<typename T, template<typename> class A >
void Array<T,A>::push_back(const T& _val)
{
	if(last==storageCapacity-1)
	{
		unsigned int newCapacity = 2*storageCapacity;
		T* newValues = Allocator::Allocate(newCapacity);

		for(unsigned int i=0;i<storageCapacity;++i)
			newValues[i] = values[i];
		
		Allocator::Desallocate(values);
		values = newValues;
		storageCapacity = newCapacity;
	}

	values[++last] = _val;
}
//-----------------------------------------------------------------------------
template<typename T, template<typename> class A >
unsigned int Array<T,A>::size() const
{
	return last+1;
}
//-----------------------------------------------------------------------------
template<typename T, template<typename> class A >
unsigned int Array<T,A>::capacity() const
{
	return storageCapacity;
}
//-----------------------------------------------------------------------------
int main()
{
	typedef Array<float,BasicAllocator> TFloatArray;
	typedef Array<int,BasicAllocator> TIntArray;

	TFloatArray fa;
	fa.push_back(1.2f);
	fa.push_back(2.3f);
	fa[0];
	fa[1];
	fa.size();
	fa.capacity();

	TIntArray fi;
	fi.push_back(1);
	fi.push_back(2);
	fi[0];
	fi[1];
	fi.size();
	fi.capacity();
	
	char c;
	std::cout << "Press any key, then enter...";
	std::cin >> c;
	return 0;
}
