//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <cassert>
#include <iostream>

const int MAX_VALUE = 3;


//-----------------------------------------------------------------------------
// Array
//-----------------------------------------------------------------------------
template<typename T>
class Array
{
	public:
		T& operator[](unsigned int _index);
		const T& operator[](unsigned int _index) const;
	private:
		T values[MAX_VALUE];
};
//-----------------------------------------------------------------------------
template<typename T>
T& Array<T>::operator[](unsigned int _index)
{
	assert(_index<MAX_VALUE);
	return values[_index];
}
//-----------------------------------------------------------------------------
template<typename T>
const T& Array<T>::operator[](unsigned int _index) const
{
	assert(_index<MAX_VALUE);
	return values[_index];
}

//-----------------------------------------------------------------------------
// Traits
//-----------------------------------------------------------------------------
template<typename T1, typename T2>
struct PromotionTraits
{	
	typedef T1 Promote;
};
//-----------------------------------------------------------------------------
template<>
struct PromotionTraits<float, int>
{	
	typedef float Promote;
};
//-----------------------------------------------------------------------------
template<>
struct PromotionTraits<double, float>
{	
	typedef double Promote;
};
//-----------------------------------------------------------------------------
template<typename T1, typename T2>
struct ArrayPromotion
{
	typedef typename PromotionTraits<T1,T2>::Promote Type;
	typedef typename Array<Type> Collection;
};
//-----------------------------------------------------------------------------
template<typename T1, typename T2>
typename ArrayPromotion<T1,T2>::Collection operator+(const Array<T1>& _t1, const Array<T2>& _t2)
{
	ArrayPromotion<T1,T2>::Collection fusion;
	for(int i=0;i<MAX_VALUE;++i)
		fusion[i] = _t1[i] + _t2[i];
	return fusion;
}

//-----------------------------------------------------------------------------
// Print
//-----------------------------------------------------------------------------
template<typename T>
void Print(Array<T> _a)
{
	for(int i=0;i<MAX_VALUE;++i)
		std::cout << i << ":" << _a[i] << " " << std::endl;
	std::cout << std::endl;
}

int main()
{
	Array<int> a1;
	a1[0] = 1;
	a1[1] = 3;
	a1[2] = 5;

	Array<int> a2;
	a2[0] = 2;
	a2[1] = 4;
	a2[2] = 6;

	Array<float> a3;
	a3[0] = 1.8f;
	a3[1] = 2.7f;
	a3[2] = 3.6f;

	Array<double> a4;
	a4[0] = 0.9;
	a4[1] = 4.5;
	a4[2] = 5.4;

	Array<int> f1 = a1 + a2;
	ArrayPromotion<double, float>::Collection f2 = a4 + a3;
	ArrayPromotion<float, int>::Collection f3 = a3 + a1;
	ArrayPromotion<double, int>::Collection f4 = a4 + a2;

	Print(f1);
	Print(f2);
	Print(f3);

	char c;
	std::cout << "Press any key, then enter...";
	std::cin >> c;
	return 0; 
}

