//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <cassert>
#include <iostream>
#include <map>
#include <vector>
#include <algorithm>
#include <string>
#include <iterator>

using namespace std;
typedef vector<int> TNoteCollection;
typedef vector<char> TLetterCollection;
typedef map<string,TNoteCollection> TStudentCollection;

//-----------------------------------------------------------------------------
// Fill students
//-----------------------------------------------------------------------------
void CreateStudents(TStudentCollection& _students)
{
	TNoteCollection s1;
	s1.push_back(15);
	s1.push_back(18);
	s1.push_back(19);

	TNoteCollection s2;
	s2.push_back(12);
	s2.push_back(11);
	s2.push_back(9);

	TNoteCollection s3;
	s3.push_back(12);
	s3.push_back(16);
	s3.push_back(15);

	_students["Dupond"] = s1;
	_students["Durand"] = s2;
	_students["Martin"] = s3;
}

//-----------------------------------------------------------------------------
// Compute mean
//-----------------------------------------------------------------------------
struct MeanFunctor
{
	MeanFunctor():mean(0.f) 
	{ 

	}
	// Mean per student
	void operator()(int _note) 
	{ 
		mean += _note;
	}
	// Mean for all students
	void operator()(const pair<string,TNoteCollection>& _notes) 
	{ 
		MeanFunctor f;
		f = for_each(_notes.second.begin(),_notes.second.end(),f);
		mean += f.mean/_notes.second.size();
	}
	float mean;
};

//-----------------------------------------------------------------------------
// Print
//-----------------------------------------------------------------------------
struct PrintFunctor
{
	// Mean for all students
	void operator()(const pair<string,TNoteCollection>& _notes) 
	{ 
		ostream_iterator<int> it(cout, " ");
		cout << "Name : " << _notes.first << " - ";
		copy(_notes.second.begin(),_notes.second.end(),it);
		cout << endl;
	}
};

//-----------------------------------------------------------------------------
// Transformation
//-----------------------------------------------------------------------------
struct LetterTransform
{
	char operator()(int _note)
	{
		return static_cast<char>((20-_note)/5 + 65);
	}
};


int main()
{
	TStudentCollection students;
	CreateStudents(students);

	// Print students' notes
	PrintFunctor p;
	for_each(students.begin(),students.end(),p);

	// Compute mean
	MeanFunctor m;
	m = for_each(students.begin(),students.end(),m);
	m.mean = m.mean/students.size();
	cout << "Mean : " << m.mean << endl;

	// Transform notes in letter notation
	for(TStudentCollection::iterator it = students.begin(); it != students.end(); ++it)
	{
		LetterTransform t;
		TLetterCollection l; 
		l.resize(it->second.size());
		transform(it->second.begin(),it->second.end(),l.begin(),t);
		ostream_iterator<char> i(cout," ");
		cout << "Name : " << it->first << " - ";
		copy(l.begin(),l.end(),i);
		cout << endl;
	}
	
	char c;
	std::cout << "Press any key, then enter...";
	std::cin >> c;
	return 0;
}
